package eu.jonahbauer.wizard.client.libgdx.actors.game.overlay;

import com.badlogic.gdx.graphics.Color;
import com.badlogic.gdx.graphics.g2d.TextureAtlas;
import com.badlogic.gdx.math.Interpolation;
import com.badlogic.gdx.scenes.scene2d.*;
import com.badlogic.gdx.scenes.scene2d.ui.Container;
import com.badlogic.gdx.scenes.scene2d.utils.TextureRegionDrawable;
import com.badlogic.gdx.utils.I18NBundle;
import eu.jonahbauer.wizard.client.libgdx.AnimationTimings;
import eu.jonahbauer.wizard.client.libgdx.UiskinAtlas;
import eu.jonahbauer.wizard.client.libgdx.WizardGame;
import eu.jonahbauer.wizard.client.libgdx.screens.GameScreen;
import lombok.AccessLevel;
import lombok.Getter;
import org.jetbrains.annotations.NotNull;

import static com.badlogic.gdx.scenes.scene2d.actions.Actions.*;

public abstract class Overlay extends Action {
    protected final GameScreen screen;
    protected final WizardGame.Data data;
    protected final I18NBundle messages;
    protected final TextureAtlas atlas;
    private long timeout;

    private Container<?> root;
    private boolean started;
    @Getter(AccessLevel.PROTECTED)
    private boolean closing;
    private boolean closed;

    public Overlay(@NotNull GameScreen gameScreen, long timeout) {
        this.screen = gameScreen;
        this.data = gameScreen.getData();
        this.messages = gameScreen.getMessages();
        this.atlas = gameScreen.getAtlas();
        this.timeout = timeout;
    }

    @Override
    public boolean act(float delta) {
        if (!started) {
            if (System.currentTimeMillis() > timeout) {
                return true;
            }

            started = true;
            show((Group) getActor());
        }

        if (System.currentTimeMillis() > timeout && !closing) {
            closing = true;
            onClosing();
        }

        return closed;
    }

    protected abstract Actor createContent();

    protected Container<?> getRoot() {
        if (root == null) {
            root = new Container<>(createContent());
            root.setBackground(new TextureRegionDrawable(data.uiskinAtlas.findRegion(UiskinAtlas.WHITE)).tint(new Color(0, 0, 0, 0.5f)));
            root.setSize(WizardGame.WIDTH, WizardGame.HEIGHT);
            root.setTouchable(Touchable.enabled);
        }

        return root;
    }

    protected void show(Group parent) {
        parent.addActor(getRoot());
    }

    protected final void onClosed() {
        if (closed) return;
        closed = true;
        getRoot().remove();
    }

    protected void onClosing() {
        getRoot().addAction(sequence(
                targeting(root, alpha(0.0f, AnimationTimings.OVERLAY_FADE, Interpolation.pow2Out)),
                run(this::onClosed)
        ));
    }

    public void close() {
        timeout = 0;
    }
}
